//  Copyright © 2012 Stephen Smally
//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation; either version 2 of the License, or
//      (at your option) any later version.
//      
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//      
//      You should have received a copy of the GNU General Public License
//      along with this program; if not, write to the Free Software
//      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
//      MA 02110-1301, USA.
//      

using Gtk;
using Lsc.Backend;
using Lsc.Utils;
using Lsc.Widgets;

namespace Lsc {
    // Elements which should be accessible to every void or class
    public enum PageType {
        HOMEPAGE,
        APPSVIEW,
        APPSINFO;
        
        public string to_string () {
            switch (this) {
                case HOMEPAGE:
                    return "Home Page";
                case APPSVIEW:
                    return "Apps View";
                case APPSINFO:
                    return "Apps Info";
                default:
                    assert_not_reached();
            }
        }
        
        public PageType[] get_all () {
            return { HOMEPAGE, APPSVIEW, APPSINFO };
        }
    }
    
    // Apps Manager which handle the PackageKit connection
    public AppsManager apps_manager;
    
    public class Frontend : Window {
        // Widgets
        public Box main_box;
        public MainToolbar toolbar;
        public PagesView pages_view;
        public ProgressInfo progress_info;
        
        // Vars
        private int _width = 0;
        
        public Frontend () {
            apps_manager = new AppsManager();
            
            destroy.connect(Gtk.main_quit);
            size_allocate.connect(on_size_allocate);
            
            window_position = WindowPosition.CENTER;
            title = "Light Software Center";
            set_default_size(600, 400);
            has_resize_grip = true;
            main_box = new Box(Orientation.VERTICAL, 0);
            
            toolbar = new MainToolbar();
            main_box.pack_start(toolbar, false, false, 0);
            
            progress_info = new ProgressInfo();
            main_box.pack_start(progress_info, false, false, 0);
            
            pages_view = new PagesView();
            
            main_box.pack_start(pages_view, true, true, 0);
            
            connect_signals();
            apps_manager.get_categories();
            
            add(main_box);
            show_all();
            
            progress_info.set_visible(false);
            pages_view.apps_info.reviews_box.set_visible(false);
            
            set_focus(null);
        }
        
        public void on_info_message_response (ResponseId id, LscApp app) {
			switch (id) {
				case ResponseId.INFO:
				    load_details (app);
				    break;
				case ResponseId.INSTALL:
				    break;
			}
		}
        
        public void load_details (LscApp app) {
			apps_manager.get_details (app);
			pages_view.set_page(PageType.APPSINFO);
		}
        
        public void load_packages (LscCategory category) {
            pages_view.set_page(PageType.APPSVIEW);
            pages_view.apps_view.apps_tree.clear();
            apps_manager.get_pkgs(category.id);
            toolbar.label.label = category.name;
            pages_view.apps_view.info_message.set_visible (false);
        }
        
        public void update_back_button (Notebook nb, Widget pg, uint page_n) {
            stdout.printf("Page->%s\n", ((PageType) page_n).to_string());
            switch ((PageType) page_n) {
                case PageType.HOMEPAGE:
                    toolbar.searchbar.set_sensitive(true);
                    toolbar.label.margin_left = 5;
                    toolbar.back.set_visible(false);
                    break;
                
                case PageType.APPSINFO:
                    toolbar.searchbar.set_sensitive(false);
                    toolbar.label.margin_left = 0;
                    toolbar.back.set_visible(true);
                    break;
                
                default:
                    toolbar.searchbar.set_sensitive(true);
                    toolbar.label.margin_left = 0;
                    toolbar.back.set_visible(true);
                    break;
            }
        }
        
        public void back_to_homepage () {
			pages_view.apps_view.apps_tree.clear();
			pages_view.set_page(PageType.HOMEPAGE);
		}
        
        public void connect_signals () {
			// When an app is added
            apps_manager.app_added.connect(pages_view.apps_view.apps_tree.append_app);
            // When something start loading
            apps_manager.loading_started.connect(on_load_started);
            // When something stop loading
            apps_manager.loading_finished.connect(on_load_finished);
            // When a category is found in the database
            apps_manager.category_added.connect(pages_view.home_page.categories_view.add_category);
            // When a Details object is found from a package
            apps_manager.details_received.connect(pages_view.apps_info.set_details);
            // When either More Info or Install is pressed
            pages_view.apps_view.info_message.choosed.connect(on_info_message_response);
            // When a category is clicked
            pages_view.home_page.categories_view.category_choosed.connect(load_packages);
            // When the page is switched
            pages_view.switch_page.connect(update_back_button);
            pages_view.switch_page.connect(toolbar.update_label);
            // When back is clicked
            toolbar.back.clicked.connect(on_back_clicked);
        }
        
        public void rework_categories_columns () {
            if (pages_view.get_current_page() != PageType.HOMEPAGE) {
                return;
            }
            if (_width != get_allocated_width() && get_allocated_width()/150 != pages_view.home_page.categories_view.columns) {
                stdout.printf("Window.width->%d rework columns\n", get_allocated_width());
                int size = get_allocated_width();
                if (size < 150 ) {
                    size = 150;
                }
                pages_view.home_page.categories_view.columns = size/150;
                
                _width = get_allocated_width();
            }
        }
        
        public void on_size_allocate (Allocation alloc) {
            rework_categories_columns();
        }
        
        public void on_back_clicked (ToolButton button) {
			switch (pages_view.get_current_page()) {
				case PageType.APPSVIEW:
				    back_to_homepage();
				    break;
				case PageType.APPSINFO:
				    pages_view.set_page (PageType.APPSVIEW);
				    pages_view.apps_view.info_message.set_visible (false);
				    break;
				default:
				    break;
			}
		}
        
        public void on_load_started (LoadingType load, string comment) {
            switch (load) {
                case LoadingType.PACKAGES:
                    progress_info.load_undefined (comment);
                    break;
                case LoadingType.DETAILS:
                    pages_view.apps_info.start_load(comment);
                    break;
                default:
                    break;
            }
        }
        
        public void on_load_finished (LoadingType load) {
            switch (load) {
                case LoadingType.CATEGORIES:
                    rework_categories_columns();
                    break;
                case LoadingType.PACKAGES:
                    progress_info.clear();
                    break;
                case LoadingType.DETAILS:
                    progress_info.clear();
                    break;
                default:
                    progress_info.clear();
                    break;
            }
        }
    }
}

int main (string[] args) {
    Gtk.init(ref args);
    new Lsc.Frontend();
    Gtk.main();
    return 0;
}
